#include "simulate.h"
#include "landuse.h"

using namespace std;

namespace Rendering {
  extern int frame;
}

namespace Map {

  LandUse land_use[size][size];
  Villager villagers[NUM_VILLAGERS];
  
  void setup_land_use() {
    for (int m = 0; m < size; ++m) {
      for (int n = 0; n < size; ++n) {
        land_use[m][n].type = EMPTY;
        if (20 <= n && n <= 40) land_use[m][n].type = TREE;
        land_use[m][n].age = 0;
        land_use[m][n].owner = 0;
      }
    }

    for (int i = 0; i < NUM_VILLAGERS; ++i) {
      Villager& V = villagers[i];
      
      V.id = i;
      V.x = i + 30.0f;
      V.y = i + 30.0f;

      float angle = 2*M_PI*((random()%1000)/1000.0f);
      V.dx = cos(angle);
      V.dy = sin(angle);
      V.hunger = 0.0f;
      V.discomfort = 0.0f;
      V.water = 0.0f;
      V.water_age = 0.0f;

      for (int j = 0; j < VILLAGER_HISTORY; ++j) {
        V.old_pos[j] = Point3d(V.x, V.y, 0.0f);
      }
    }
  }

  void simulate_villagers() {
    // TODO: we need to keep track of which grid position this
    // villager is in; how do we update that variable (without doing
    // it every frame)?  Maybe measure every N frames + every time we
    // walk outside grid center + radius...
    
    const float dt = 0.01f;
    for (int i = 0; i < NUM_VILLAGERS; ++i) {
      Villager& V = villagers[i];
      pair<int,int> grid_location = world_to_grid(V.x, V.y);
      if (Map::valid(grid_location.first, grid_location.second)) {
        int m = grid_location.first, n = grid_location.second;

        if (V.water > 0.0f) {
          V.water_age += 1.0f;
        } else {
          V.water_age -= 1.0f;
        }
        
        // Flow could be positive (map to villager) or negative (villager to map)
        float flow = Map::water[m][n] - V.water;
        if (flow > 0.0f && V.water_age < -500.0f) {
          V.water_age = -600.0f;
          if (flow > +0.05f) flow = +0.05f;
          Map::water[m][n] -= flow;
          V.water += flow;
          if (V.water >= 0.5f) {
            // Reverse direction when full
            V.dx = -V.dx;
            V.dy = -V.dy;
          }
        } else if (flow < 0.0f && V.water_age > 500.0f) {
          if (flow < -0.01f) flow = -0.01f;
          Map::water[m][n] -= flow;
          V.water += flow;
          if (V.water == 0.0f) {
            // Reverse direction when empty
            V.dx = -V.dx;
            V.dy = -V.dy;
            V.water_age = 0.0f;
          }
        }
      } else {
        // The villager is out of bounds.  Teleport him to the
        // location of the previous villager.
        Villager& other = villagers[(i+NUM_VILLAGERS-1) % NUM_VILLAGERS];
        V.x = other.x + 1.0f;
        V.y = other.y + 1.0f;
      }
      
      if (Rendering::frame % 10 == 0) {
        for (int j = VILLAGER_HISTORY-1; j > 0; --j) {
          V.old_pos[j] = V.old_pos[j-1];
        }
      }
      V.old_pos[0] = Point3d(V.x, V.y, 0.0f);// z set in game.cc :-(
      
      V.x += V.dx * dt;
      V.y += V.dy * dt;

      // Accelerate in a random direciton
      float angle = 2*M_PI*((random()%1000)/1000.0f);
      V.dx += cos(angle) * dt;
      V.dy += sin(angle) * dt;

      // Accelerate depending on the next villager's location
      if (i > 0) {
        Villager& other = villagers[i-1];
        float dx = other.x - V.x;
        float dy = other.y - V.y;
        float distance2 = dx*dx + dy*dy;
        if (distance2 > 4.0) {
          // We're far away.  Move closer.
          V.dx += dt * dx * 0.1f * (distance2 - 4.0f);
          V.dy += dt * dy * 0.1f * (distance2 - 4.0f);
        } else if (distance2 < 1.0) {
          // We're too close.  Move away
          V.dx -= 0.2 * dt * dx;
          V.dy -= 0.2 * dt * dy;
        }
      }

      // Try to move in a certain speed range
      float speed = sqrt(V.dx*V.dx + V.dy*V.dy);
      if (speed < 5.0f) {
        V.dx *= 1.1f;
        V.dy *= 1.1f;
      } else if (speed > 10.0f) {
        V.dx *= (10.0f / speed);
        V.dy *= (10.0f / speed);
      }
    }
  }
}
