  /* Reaction-diffusion pattern.  This could be useful for generating textures.
   How can we adapt it to use hexagons? */
  void reactionDiffusion() {
    const int W = 64, H = 64;
    static double A[W][H] = {0.5};
    static double B[W][H] = {0.5};
    static double s = 0.2/16.0;
    static int frame = 0;
    if (++frame == 1) {
      for (int i = 0; i < W; ++i) {
        for (int j = 0; j < H; ++j) {
          A[i][j] = 4 + 0.01*(random()%200-random()%200);
          B[i][j] = 4 + 0.01*(random()%200-random()%200);
        }
      }
    }

    double da[W][H], db[W][H];
    for (int i = 0; i < W; ++i) {
      for (int j = 0; j < H; ++j) {
        double C = 12 + 0.1*((3*i+5*j)%300);
        da[i][j] = s*(16-A[i][j]*B[i][j])
                   + 0.5*0.25*(A[(i+1)%W][j] + A[(i-1+W)%W][j]
                               + A[i][(j+1)%H] + A[i][(j-1+H)%H]
                               - 4*A[i][j]);
        db[i][j] = s*(A[i][j]*B[i][j] - B[i][j] - C)
                   + 0.5*0.0625*(B[(i+1)%W][j] + B[(i-1+W)%W][j]
                                 + B[i][(j+1)%H] + B[i][(j-1+H)%H]
                                 - 4*B[i][j]);
      }
    }

    A[W/2][H/4] = 4;
    B[W/2][H/4] = 0;
    A[W/4][H/2] = 0;
    B[W/4][H/2] = 4;
    double speed = 0.01;
    for (int i = 0; i < W; ++i) {
      for (int j = 0; j < H; ++j) {
        A[i][j] = max(min(8.0, A[i][j]+speed*da[i][j]), 0.0);
        B[i][j] = max(min(8.0, B[i][j]+speed*db[i][j]), 0.0);
        glColor3f(0.12*A[i][j], 0.0, 0.12*B[i][j]);
        glBegin(GL_POLYGON);
        glVertex3f(i, j, 5);
        glVertex3f(i+1, j, 5);
        glVertex3f(i+1, j+1, 5);
        glVertex3f(i, j+1, 5);
        glEnd();
      }
    }
  }

