#include "include_gl.h"

#include "world.h"
#include "rendering.h"

namespace Rendering {
  // These two externs -- HACK:
  extern int triangle_count;
  extern array x, y;
  
  void _draw_hexagon(int m, int n, Rendering::array& z, bool outline) {
    glPushName(m);
    glPushName(n);
    
    // Render CENTER + 12 directions for a total of 12 triangles
    float vertices[14*3];
    float* v = vertices;
    glVertexPointer(3, GL_FLOAT, 0, vertices);
    glEnableClientState(GL_VERTEX_ARRAY);

    if (!outline) {
      *v++ = x[m][n]; *v++ = y[m][n]; *v++ = z[m][n];
    }
    for (int k = 1; k <= 7; ++k) {
      int k1 = (k==7)? 1 : k;

      // First, pick the midpoint between the center and the hexagon in direction k
      int m1 = m + Hex::dm[k1], n1 = n + Hex::dn[m&1][k1];
      *v++ = x[m][n] + 0.5f*Hex::neighbor_dx[k1];
      *v++ = y[m][n] + 0.5f*Hex::neighbor_dy[k1];
      if (!Map::valid(m1, n1)) {
        *v++ = z[m][n];
      } else {
        *v++ = 0.5*(z[m][n] + z[m1][n1]);

        // Then pick a vertex of this hexagon, which is also shared by two others
        int k2 = (k>=6)? 1 : (k+1);
        int m2 = m + Hex::dm[k2], n2 = n + Hex::dn[m&1][k2];

        if (k < 7 && Map::valid(m2, n2)) {
          *v++ = (x[m][n] + x[m1][n1] + x[m2][n2])/3;
          *v++ = (y[m][n] + y[m1][n1] + y[m2][n2])/3;
          *v++ = (z[m][n] + z[m1][n1] + z[m2][n2])/3;
        }
      }
    }

    assert(unsigned(v-vertices) <= sizeof(vertices)/sizeof(vertices[0]));
    glDrawArrays(outline? GL_LINE_STRIP:GL_TRIANGLE_FAN, 0, (v-vertices)/3);
    if (!outline) {
      Rendering::triangle_count += ((v-vertices)/3 - 2);
    }
    glDisableClientState(GL_VERTEX_ARRAY);
    
    glPopName();
    glPopName();
  }

  void draw_outline_hexagon(int m, int n, Rendering::array& z) {
    _draw_hexagon(m, n, z, true);
  }

  void draw_solid_hexagon(int m, int n, Rendering::array& z) {
    _draw_hexagon(m, n, z, false);
  }
}

  
