#include <time.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <math.h>
#include "include_gl.h"

// TODO: I broke this code when abstracting out into main.cc.  Fix?

struct vec3d {
  double x, y, z;
  vec3d(): x(0), y(0), z(0) {}
  vec3d(double x_, double y_, double z_): x(x_), y(y_), z(z_) {}

  double length() { return sqrt(x*x+y*y+z*z); }
};

// NOTE: through experimentation, I found that it was reasonable to
// assume that after length() > 100, the object is no longer visible
// on the screen.
struct Query {
  // An object representing a query
  vec3d position;
  vec3d velocity;
  vec3d rotation; float u, du; // rotation angle
  float size; // object size
  float r, g, b, a;
  
  Query()
    :rotation(rand() % 30 - 15, rand() % 30 - 15, rand() % 30 - 15), u(0),
     size(1) {

    double latency = 0.1;
    for(int i = 0; i < 3; ++i) {
      latency += (rand() % 400) / 200.0;
    }
    
    double speed = 1/latency;
    static double _angle = 0;
    double angle = (rand() % 100) / 100.0;
    angle = _angle - floor(_angle); _angle += 0.01;
    position = vec3d(0, -40, 0);
    velocity = vec3d(40*speed*cos(angle*6.28), 90, 40*speed*sin(angle*6.28));

    double redness = speed;
    if(redness < 0) redness = 0;
    if(redness > 1) redness = 1;
    r = 1.0 - 0.5*redness;
    g = 0.1 + 0.8*redness;
    b = 0.5 - (r+g)*0.25;
    a = 1.0;

    du = 350 + 850*speed;
  }

  void tick(float dt) {
    u += du*dt;
    if ( u > 360 ) u = 0;
    position.x += dt * velocity.x;
    position.y += dt * velocity.y;
    position.z += dt * velocity.z;

    velocity.y -= dt * 50;

    if ( a > 0.05 ) a -= 0.003;
  }
  
  void draw() {
    glPushMatrix();
    glTranslatef(position.x, position.y, position.z);
    glRotatef(u, rotation.x, rotation.y, rotation.z);
    glColor4f(r, g, b, 1.0);

#if 1
    glutSolidCube(size);
#else
    glBegin(GL_POLYGON);
    glNormal3f(0.0, 1.0, 0.0);
    glTexCoord2f(0.0, 0.0); glVertex3f(size/2, 0.0, size/2);
    glTexCoord2f(0.0, 1.0); glVertex3f(size/2, 0.0, -size/2);
    glTexCoord2f(1.0, 1.0); glVertex3f(-size/2, 0.0, -size/2);
    glTexCoord2f(1.0, 0.0); glVertex3f(-size/2, 0.0, size/2);
    glEnd();

    glColor3f(1-r, 1-g, 1-b);
    glBegin(GL_POLYGON);
    glNormal3f(0.0, 1.0, 0.0);
    glTexCoord2f(1.0, 1.0); glVertex3f(size/2, 0.0, size/2);
    glTexCoord2f(1.0, 0.0); glVertex3f(size/2, 0.0, -size/2);
    glTexCoord2f(0.0, 0.0); glVertex3f(-size/2, 0.0, -size/2);
    glTexCoord2f(0.0, 1.0); glVertex3f(-size/2, 0.0, size/2);
    glEnd();
#endif
 
    glPopMatrix();
  }

  bool finished() {
    return position.length() > 100;
  }
};

void paint() {
  static Query q[500];
  static double last_time = 0.0;

  /* Actual interesting code */
  timeval tv;
  gettimeofday(&tv, NULL);
  double current_time = double(tv.tv_sec) + double(tv.tv_usec) * 1e-6;
  if ( last_time == 0.0 )
    last_time = current_time;

  float dt = current_time - last_time;
  last_time = current_time;
  
  /* draw something ... */
  for(int i=0; i<sizeof(q)/sizeof(q[0]); i++) {
    q[i].tick(dt);
    q[i].draw();
    if(q[i].finished())
      q[i] = Query();
  }
}

// Factory function
#include "main.h"
GlApp* getGlAppInstance() {
  return new GlApp;
}
